﻿using System;
//using System.Collections.Generic;
//using System.ComponentModel;
//using System.Data;
//using System.Drawing;
//using System.Linq;
using System.Text;
using System.Windows.Forms;
using DeVaSys.UsbI2cIo.CS.Pinvoke;

namespace Vcs2008PinvokeExample
{
    public partial class frmMain : Form
    {

        StringBuilder sbOutput = new StringBuilder("", 2048);
        public IntPtr hDevInstance;
        public String sDevSymName = "UsbI2cIo";
        public IntPtr INVALID_HANDLE_VALUE = (IntPtr)(-1);

        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
        }

        private void CloseDiHandle()
        {
            if (hDevInstance != INVALID_HANDLE_VALUE)
            {
                DAPI.CloseDeviceInstance(hDevInstance);
                hDevInstance = INVALID_HANDLE_VALUE;
            }
        }

        private bool OpenDiHandle()
        {
            CloseDiHandle();
            hDevInstance = DAPI.OpenDeviceInstance(sDevSymName, 0);

            if (hDevInstance != INVALID_HANDLE_VALUE)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private void DoAll()
        {
            sbOutput.Length = 0;
            DemoGetDllAndDriverVersions();
            sbOutput.AppendLine();
            if (DemoGetDeviceCount() > 0)
            {
                sbOutput.AppendLine();
                DemoGetDeviceInfo();
                sbOutput.AppendLine();
                if (OpenDiHandle())
                {
                    sbOutput.AppendLine("Opened handle to UsbI2cIo board");
                    sbOutput.AppendLine();
                    DemoGetSerialId();
                    sbOutput.AppendLine();
                    DemoIo();
                    sbOutput.AppendLine();
                    DemoI2c();
                    sbOutput.AppendLine();
                    DemoProperties();
                    sbOutput.AppendLine();
                    DemoGetVendorRequest();
                    sbOutput.AppendLine();
                    DemoGetAndClearError();
                    sbOutput.AppendLine();
                    CloseDiHandle();
                    sbOutput.AppendLine("Closed handle to UsbI2cIo board");
                }
                else
                {
                    sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board");
                }
            }
            txtSbOut.Text = sbOutput.ToString();
        }
        
        private bool DemoDetectDevice()
        {
            sbOutput.AppendLine("DemoDetectDevice");

            if (DAPI.DetectDevice(hDevInstance))
            {
                sbOutput.AppendLine("UsbI2cIo device detected");
                return true;
            }
            else
            {
                sbOutput.AppendLine("UsbI2cIo device not detected");
                return false;
            }
        }

        private byte DemoGetDeviceCount()
        {
            byte byCount;
            
            sbOutput.AppendLine("DemoGetDeviceCount");

            byCount = DAPI.GetDeviceCount(sDevSymName);
            sbOutput.Append("Attached UsbI2cIo Device(s) = ");
            sbOutput.AppendLine(byCount.ToString());
            return byCount;
        }

        private void DemoGetDllAndDriverVersions()
        {
            UInt16 Version;

            sbOutput.AppendLine("DemoGetDllAndDriverVersions");

            Version = DAPI.GetDllVersion();
            sbOutput.AppendFormat("UsbI2cio DLL Version    = {0:X}.{1:X2}", (byte)(Version/256), (byte)Version);
            sbOutput.AppendLine();

            if( Version < 0x0304)
            {
                MessageBox.Show("You must use version 3.04 (or later) of usbi2cio.dll file in order to properly fetch serial ID from version C USB-I2C/IO boards",
                    "Warning: Old version of UsbI2cIo.dll detected!",
                    MessageBoxButtons.OK);
            }

            Version = DAPI.GetDriverVersion();
            sbOutput.AppendFormat("UsbI2cio Driver Version = {0:X}.{1:X2}", (byte)(Version / 256), (byte)Version);
            sbOutput.AppendLine();
        }

        private void DemoGetFirmwareVersion()
        {
            UInt16 Version;

            sbOutput.AppendLine("DemoGetFirmwareVersion");

            if (DAPI.GetFirmwareVersion(hDevInstance, out Version))
            {
                sbOutput.AppendFormat("    Firmware Version = {0:X}.{1:X2}", (byte)(Version / 256), (byte)Version);
                sbOutput.AppendLine();
            }
            else
            {
                sbOutput.AppendLine("    Error: Firmware Version = Unavailable");
            }
        }

        private void DemoGetSerialId()
        {
            StringBuilder sbSerialId = new StringBuilder("", 9);
            
            sbOutput.AppendLine("DemoGetSerialId");

            if(DAPI.GetSerialId(hDevInstance, sbSerialId)) {
                sbOutput.Append("Serial ID = ");
                sbOutput.AppendLine(sbSerialId.ToString());
            }
            else {
                sbOutput.AppendLine("Error: Serial ID = Unavalable");
            }
        }

        private void DemoGetDeviceInfo()
        {
            DAPI.DEV_INFO_ARRAY DeviceInfoArray = new DAPI.DEV_INFO_ARRAY();
            byte byCount = 0;
            byte i;

            sbOutput.AppendLine("DemoGetDeviceInfo");

            byCount = DAPI.GetDeviceInfo(sDevSymName, ref DeviceInfoArray);
            if (byCount > 0)
            {
                sbOutput.AppendLine("Device Info:");
                for( i = 0; i<(byCount); i++) {
                    sbOutput.Append("  Device Instance = ");
                    sbOutput.Append(DeviceInfoArray.DevInfo[i].byInstance.ToString());
                    sbOutput.Append(", SerialId = ");
                    sbOutput.AppendLine(DeviceInfoArray.DevInfo[i].SerialId.ToString());
                    DemoOpenDeviceBySerialId(DeviceInfoArray.DevInfo[i].SerialId.ToString());
                }
            }
            else {
                sbOutput.AppendLine("Error: Device Info = Unavalable");
            }
        }

        private bool DemoOpenDeviceBySerialId(String SerialId)
        {
            sbOutput.AppendLine("DemoOpenDeviceBySerialId");

            // now let's close our handle
            CloseDiHandle();

            // get a handle to a board, using the SerialId of the board
            hDevInstance = DAPI.OpenDeviceBySerialId(sDevSymName, SerialId);

            // test result of function call and flag success or failure
            if(hDevInstance != INVALID_HANDLE_VALUE) {
                sbOutput.Append("    Opened handle to device using Serial ID ");
                sbOutput.AppendLine(SerialId);
                DemoGetFirmwareVersion();
                return true;
            }
            else {
                sbOutput.Append("    Error: Failed to open device using Serial ID ");
                sbOutput.AppendLine(SerialId);
                return false;
            }
        }

        private void DemoIo()
        {

            UInt32 IoPortConfig = 0x00000000;
            UInt32 IoPortMask = 0x00000000;
            UInt32 IoPortData = 0x00000000;

            sbOutput.AppendLine("DemoIo");

            if (DAPI.GetIoConfig(hDevInstance, out IoPortConfig))
            {
                sbOutput.AppendFormat("Read I/O Configuration 0x{0:X8} from UsbI2cIo board", IoPortConfig);
                sbOutput.AppendLine();
            }
            else
            {
                sbOutput.AppendLine("Error I/O Configuration = Unavalable");
            }

            IoPortConfig = 0x00;
            if (DAPI.ConfigIoPorts(hDevInstance, IoPortConfig))
            {
                sbOutput.AppendFormat("Wrote I/O Configuration 0x{0:X8} to UsbI2cIo board", IoPortConfig);
                sbOutput.AppendLine();
            }
            else
            {
                sbOutput.AppendLine("Error: Failed to write I/O Configuration to UsbI2cIo board");
            }


            if (DAPI.GetIoConfig(hDevInstance, out IoPortConfig))
            {
                sbOutput.AppendFormat("Read I/O Configuration 0x{0:X8} from UsbI2cIo board", IoPortConfig);
                sbOutput.AppendLine();
            }
            else
            {
                sbOutput.AppendLine("Error I/O Configuration = Unavalable");
            }

            if (DAPI.ReadIoPorts(hDevInstance, out IoPortData))
            {
                sbOutput.AppendFormat("Read I/O Data 0x{0:X8} from UsbI2cIo board", IoPortData);
                sbOutput.AppendLine();
            }
            else
            {
                sbOutput.AppendLine("Error: I/O Data = Unavalable");
            }

            //  IoPortMask = &HFFFFFFFF // this produces an error
            IoPortMask = 4294967295;    //' have to do it this way because VB oddity
            IoPortData = 0x01;
            if (DAPI.WriteIoPorts(hDevInstance, IoPortData, IoPortMask))
            {
                sbOutput.AppendFormat("Wrote I/O Data 0x{0:X8}, I/O Mask 0x{1:X8} to UsbI2cIo board", IoPortData, IoPortMask);
                sbOutput.AppendLine();
            }
            else
            {
                sbOutput.AppendLine("Error: Failed to write I/O Data and I/O Mask to UsbI2cIo board");
            }

            if (DAPI.ReadIoPorts(hDevInstance, out IoPortData))
            {
                sbOutput.AppendFormat("Read I/O Data 0x{0:X8} from UsbI2cIo board", IoPortData);
                sbOutput.AppendLine();
            }
            else
            {
                sbOutput.AppendLine("Error: I/O Data = Unavalable");
            }
        }

        private void DemoI2c()
        {

            DAPI.I2C_TRANS I2cTrans = new DAPI.I2C_TRANS();
            Int32 lResult;
            int i;

            sbOutput.AppendLine("DemoI2c");

            I2cTrans.bySlvDevAddr = 0xA2;
            I2cTrans.byTransType = DAPI.I2C_TRANS_TYPE.ADR_16;
            I2cTrans.wMemoryAddr = 0x3F00;
            I2cTrans.wCount = 0x0008; 


            sbOutput.AppendLine("Setting up I2C_TRANS structure for call to ReadI2c():");
            sbOutput.AppendFormat("  I2cTrans.bySlvDevAddr: 0x{0:X2}", I2cTrans.bySlvDevAddr);
            sbOutput.AppendLine();
            sbOutput.AppendFormat("  I2cTrans.byTransType: 0x{0:X2}", (byte)I2cTrans.byTransType);
            sbOutput.AppendLine();
            sbOutput.AppendFormat("  I2cTrans.wMemoryAddr: 0x{0:X4}", I2cTrans.wMemoryAddr);
            sbOutput.AppendLine();
            sbOutput.AppendFormat("  I2cTrans.wCount: 0x{0:X4}", I2cTrans.wCount);
            sbOutput.AppendLine();

            lResult = DAPI.ReadI2c(hDevInstance, ref I2cTrans);
            if(lResult == I2cTrans.wCount) {
                sbOutput.AppendFormat("ReadI2c call was successful, return value = 0x{0:X8}", lResult);
                sbOutput.AppendLine();
                sbOutput.Append("  I2cTrans.Data(): {");
                for(i = 0; i < lResult; i++) {
                    sbOutput.AppendFormat(" 0x{0:X2}", I2cTrans.Data[i]);
                }
                sbOutput.AppendLine(" }");
            }
            else {
                sbOutput.AppendFormat("Error: ReadI2c call failed, return value = 0x{0:X8}\r\n", lResult);
            }
        }


        private void DemoProperties()
        {

            byte byPropertyCount = 0;
            byte byProperty = 0;
            byte i;
//            Int32 lValue;
//            int iValue;
//            byte byValue;

            sbOutput.AppendLine("DemoProperties");

            // Get Property Table Size
            if (DAPI.GetProperty(hDevInstance, out byPropertyCount, DAPI.PROPERTY_IDX.WRCOMMAND_RDCOUNT))
            {
                sbOutput.AppendFormat("Property Count: 0x{0:X2} ({1})", byPropertyCount, byPropertyCount);
                sbOutput.AppendLine();

                // Get all properties
                for (i = 0; i < byPropertyCount; i++)
                {
                    if (DAPI.GetProperty(hDevInstance, out byProperty, (DAPI.PROPERTY_IDX)i))
                    {
                        sbOutput.AppendFormat("Property 0x{0:X2} ({1})", byProperty, byProperty);
                        sbOutput.AppendLine();
                    }
                    else
                    {
                        sbOutput.AppendLine("Error: GetProperty call failed");
                    }
                }

                // Set a property
                //if(DAPI.SetProperty(hDevInstance, PROPERTY_WRCOMMAND_RDCOUNT, PROPERTY_CMD_RESET_TO_DEFAULTS)) {
                //    if(DAPI.SetProperty(hDevInstance, PROPERTY_WRCOMMAND_RDCOUNT, PROPERTY_CMD_STORE_TABLE_TO_EEPROM)) {
                //    }
                //}

                // Make current properties non-volatile (store them in flash)
                // Invalidate flash copy of properties (restores default properties)
            }
            else
            {
                sbOutput.AppendLine("Error: GetProperty call failed");
            }
        }

        private void DemoGetVendorRequest()
        {

            byte[] VrData = new byte[DAPI.VR_MAX_DATA];
            byte byRequest = 0xF2;
            UInt16 wValue = 0xA202;
            UInt16 wIndex = 0x0000;
            UInt16 wLength = 0x0400;
            int i;

            sbOutput.AppendLine("DemoVendorRequest");

            // bmRequestType = Device to Host (Get), Vendor Request 
            // bRequest = 0xF2
            // wValue = I2C_TRANS.byTransType (byte 2) and I2C_TRANS.bySlvDevAddr (byte 3)
            // wIndex = I2C_TRANS.wMemoryAddr
            // wLength = I2C_TRANS.wCount

            if(DAPI.GetVendorRequest(hDevInstance, ref VrData[0], byRequest, wValue, wIndex, wLength) == wLength) {
                sbOutput.AppendLine("GetVendorRequest call was successful");
                sbOutput.Append("Data(): {");
                for(i = 0; i < wLength; i++) {
                    sbOutput.AppendFormat(" 0x{0:X2}", VrData[i]);
                }
                sbOutput.AppendLine(" }");
            }
            else {
                sbOutput.AppendLine("Error: GetVendorRequest call failed");
            }
        }

        private void DemoGetAndClearError()
        {
            Int32 iError = 0;

            sbOutput.AppendLine("DemoGetAndClearError");

            if(DAPI.GetLastFirmwareError(hDevInstance, out iError)) {
                sbOutput.AppendFormat("GetLastFimrwareError call returned: 0x{0:X8}", iError);
                sbOutput.AppendLine();
                // only least significant byte of returned unsigned integer is used, need to strip upper bytes
                if( (iError & 0x000000FF) > 0x00) {
                    sbOutput.AppendFormat("Last Firmware Error : 0x{0:X2}", iError & 0xFF);
                    sbOutput.AppendLine();
                }
                else {
                    sbOutput.AppendLine("No Firmware Error : 0x00");
                }
            }
            else {
                sbOutput.AppendLine("Error: GetLastFirmwareError call failed");
            }
        }


        private void btnDoAll_Click(object sender, EventArgs e)
        {
            DoAll();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            sbOutput.Length = 0;
            txtSbOut.Text = sbOutput.ToString();
        }

        private void btnVersion_Click(object sender, EventArgs e)
        {
            DemoGetDllAndDriverVersions();
            sbOutput.AppendLine();
            if( DemoGetDeviceCount() > 0 )
            {
                DemoGetDeviceInfo();
            }
            CloseDiHandle();    // now let's close our handle
            sbOutput.AppendLine();
            txtSbOut.Text = sbOutput.ToString();
        }

        private void btnIo_Click(object sender, EventArgs e)
        {
            if(OpenDiHandle()) {
                sbOutput.AppendLine("Opened handle to UsbI2cIo board");
                DemoIo();
                CloseDiHandle();
                sbOutput.AppendLine("Closed handle to UsbI2cIo board");
            }
            else {
                sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board");
            }
            sbOutput.AppendLine();
            txtSbOut.Text = sbOutput.ToString();
        }

        private void btnI2c_Click(object sender, EventArgs e)
        {
            if(OpenDiHandle()) {
                sbOutput.AppendLine("Opened handle to UsbI2cIo board");
                DemoI2c();
                CloseDiHandle();
                sbOutput.AppendLine("Closed handle to UsbI2cIo board");
            }
            else {
                sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board");
            }
            sbOutput.AppendLine();
            txtSbOut.Text = sbOutput.ToString();
        }

        private void btnProperties_Click(object sender, EventArgs e)
        {
            if(OpenDiHandle()) {
                sbOutput.AppendLine("Opened handle to UsbI2cIo board");
                DemoProperties();
                CloseDiHandle();
                sbOutput.AppendLine("Closed handle to UsbI2cIo board");
            }
            else {
                sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board");
            }
            sbOutput.AppendLine();
            txtSbOut.Text = sbOutput.ToString();
        }

        private void btnVenReqs_Click(object sender, EventArgs e)
        {
            if (OpenDiHandle())
            {
                sbOutput.AppendLine("Opened handle to UsbI2cIo board");
                DemoGetVendorRequest();
                CloseDiHandle();
                sbOutput.AppendLine("Closed handle to UsbI2cIo board");
            }
            else
            {
                sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board");
            }
            sbOutput.AppendLine();
            txtSbOut.Text = sbOutput.ToString();

        }

        private void btnGetClrError_Click(object sender, EventArgs e)
        {
            if (OpenDiHandle())
            {
                sbOutput.AppendLine("Opened handle to UsbI2cIo board");
                DemoGetAndClearError();
                CloseDiHandle();
                sbOutput.AppendLine("Closed handle to UsbI2cIo board");
            }
            else
            {
                sbOutput.AppendLine("Error: Failed to open handle to UsbI2cIo board");
            }
            sbOutput.AppendLine();
            txtSbOut.Text = sbOutput.ToString();

        }

        private void cmdExit_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
