using System;
using System.Runtime.InteropServices;

namespace DeVaSys.UsbI2cIo.CS.Pinvoke
{
    public class DAPI
    {
        /// USBI2CIO_MAX_DEVICES -> 127
        public const int MAX_DEVICES = 127;

        /// USBI2CIO_VR_MAX_DATA -> 1088
        public const int VR_MAX_DATA = 1088;

        /// USBI2CIO_I2C_MAX_DATA -> USBI2CIO_VR_MAX_DATA
        public const int I2C_MAX_DATA = VR_MAX_DATA;

        /// USBI2CIO_IO_MAX_DATA -> USBI2CIO_VR_MAX_DATA
        public const int IO_MAX_DATA = VR_MAX_DATA;

        public enum I2C_TRANS_TYPE : byte
        {
            // standard transaction types
            NOADR = 0x00,         // read or write with no address cycle
            ADR_8,                 // read or write with 8 bit address cycle
            ADR_16,                // read or write with 16 bit address cycle
            NOADR_NS,             // read or write with no address cycle, stop signaling inhibited
            XICOR,                // read or write with 8 bit instruction cycle, non I2C compliant use of R/W bit

            // customer specific transactions types for accessing rams, only write is mode supported
            ADR_8_NONSEQ = 0x30,   // read or write with 8 bit address cycle, non sequential
            ADR_16_NONSEQ,         // read or write with 16 bit address cycle, non sequential
            ADR_24_NONSEQ          // read or write with 24 bit address cycle, non sequential
        }

        public enum PROPERTY_IDX : byte
        {
            // WRITE COMMAND, READ MAX PROPERTIES (byte 0)
            WRCOMMAND_RDCOUNT = 0x00,          // when setting, indicates command, when getting, fetches table size
            // I2C PROPERTIES (byte 1)
            I2C_CONFIG,
            // FAST TRANSFER PROPERTIES (byte 2)
            FAST_XFER_CONFIG,
            // I/O PINS GLOBAL PROPERTIES (byte 3)
            IO_CONFIG_GLOBAL,
            // I/O PINS PORT CONFIG PROPERTIES (bytes 4-6)
            IO_CONFIG_PORTA,
            IO_CONFIG_PORTB,
            IO_CONFIG_PORTC,
            // I/O PINS PORT OUTPUT PROPERTIES (bytes 7-9)
            IO_OUTPUT_PORTA,
            IO_OUTPUT_PORTB,
            IO_OUTPUT_PORTC,
            // DEBUG PROPERTIES (bytes 10-12)
            DEBUG_CONFIG_GLOBAL,
            DEBUG_CONFIG_0,
            DEBUG_CONFIG_1,
            // USER PROPERTIES (bytes 13-16)
            USER_0,
            USER_1,
            USER_2,
            USER_3,
            // I/O PINS
            IO_CONFIG_PORTD,
            IO_OUTPUT_PORTD,
            // OUTPUT MODE (Pusb-Pull or Open-Drain)
            IO_OUTPUT_MODE_PORTA,
            IO_OUTPUT_MODE_PORTB,
            IO_OUTPUT_MODE_PORTC,
            IO_OUTPUT_MODE_PORTD,
            // I2C DEFAULT CHANNEL SELECTION AND CLOCK SPEED
            I2C_DEFAULT_CHANNEL,
            I2C_CHAN0_CLK_LO,
            I2C_CHAN0_CLK_HI,
            I2C_CHAN1_CLK_LO,
            I2C_CHAN1_CLK_HI,
            I2C_CHAN2_CLK_LO,
            I2C_CHAN2_CLK_HI,
            MAX_PROPERTY_BYTES
        }

        public enum PROPERTY_CMD : byte
        {
            STORE_TABLE_TO_EEPROM = 0x00,
            LOAD_TABLE_FROM_EEPROM,
            DISABLE_EEPROM_TABLE,
            RESET_TO_DEFAULTS
        }

        [StructLayoutAttribute(LayoutKind.Sequential)]
        public struct DEV_INFO
        {
            public byte byInstance;

            [MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst = 9)]
            public string SerialId;
        }

        [StructLayoutAttribute(LayoutKind.Sequential)]
        public struct DEV_INFO_ARRAY
        {
            /// DEV_INFO[USBI2CIO_MAX_DEVICES]
            [MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = DAPI.MAX_DEVICES, ArraySubType = UnmanagedType.Struct)]
            public DEV_INFO[] DevInfo;
        }

        [StructLayoutAttribute(LayoutKind.Sequential)]
        public struct I2C_TRANS
        {
            /// BYTE->unsigned char
            public I2C_TRANS_TYPE byTransType;

            /// BYTE->unsigned char
            public byte bySlvDevAddr;

            /// WORD->unsigned short
            public UInt16 wMemoryAddr;

            /// WORD->unsigned short
            public UInt16 wCount;

            /// BYTE[1088]
            [MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst = DAPI.I2C_MAX_DATA, ArraySubType = UnmanagedType.I1)]
            public byte[] Data;
        }

    // WORD DAPI_GetDllVersion(void);

        /// Return Type: WORD->unsigned short
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_GetDllVersion")]
        public static extern UInt16 GetDllVersion();

    // WORD  DAPI_GetDriverVersion(void);

        /// Return Type: WORD->unsigned short
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_GetDriverVersion")]
        public static extern UInt16 GetDriverVersion();

    // BOOL  DAPI_GetFirmwareVersion(IN HANDLE hDevInstance, OUT LPWORD lpwVersion);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        ///lpwVersion: LPWORD->WORD*
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_GetFirmwareVersion")]
        [return: MarshalAsAttribute(UnmanagedType.Bool)]
        public static extern bool GetFirmwareVersion(System.IntPtr hDevInstance, out UInt16 lpwVersion);

    // HANDLE DAPI_OpenDeviceInstance(IN LPCSTR lpsDevName, IN BYTE byDevInstance);

        /// Return Type: HANDLE->void*
        ///lpsDevName: LPCSTR->CHAR*
        ///byDevInstance: BYTE->unsigned char
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_OpenDeviceInstance")]
        public static extern System.IntPtr OpenDeviceInstance([InAttribute()]
            [MarshalAsAttribute(UnmanagedType.LPStr)] string lpsDevName,
            byte byDevInstance);

    // BOOL  DAPI_CloseDeviceInstance(HANDLE hDevInstance);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_CloseDeviceInstance")]
        [return: MarshalAsAttribute(UnmanagedType.Bool)]
        public static extern bool CloseDeviceInstance(System.IntPtr hDevInstance);

    // BOOL DAPI_DetectDevice(IN HANDLE hDevInstance);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_DetectDevice")]
        [return: MarshalAsAttribute(UnmanagedType.Bool)]
        public static extern bool DetectDevice(System.IntPtr hDevInstance);

    // BYTE DAPI_GetDeviceCount(IN LPCSTR lpsDevName);

        /// Return Type: BYTE->unsigned char
        ///lpsDevName: LPCSTR->CHAR*
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_GetDeviceCount")]
        public static extern byte GetDeviceCount([InAttribute()]
            [MarshalAsAttribute(UnmanagedType.LPStr)] string lpsDevName);

    // BYTE DAPI_GetDeviceInfo(IN LPCSTR lpsDevName, OUT LPDEV_INFO_ARRAY lpDevInfoArray);

        /// Return Type: BYTE->unsigned char
        ///lpsDevName: LPCSTR->CHAR*
        ///lpDevInfoArray: LPDEV_INFO_ARRAY->_DEV_INFO_ARRAY*
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_GetDeviceInfo")]
        public static extern byte GetDeviceInfo([InAttribute()]
            [MarshalAsAttribute(UnmanagedType.LPStr)] string lpsDevName,
            ref DEV_INFO_ARRAY lpDevInfoArray);
        
    // HANDLE DAPI_OpenDeviceBySerialId(IN LPCSTR lpsDevName, IN LPCSTR lpsDevSerialId);

        /// Return Type: HANDLE->void*
        ///lpsDevName: LPCSTR->CHAR*
        ///lpsDevSerialId: LPCSTR->CHAR*
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_OpenDeviceBySerialId")]
        public static extern System.IntPtr OpenDeviceBySerialId([InAttribute()]
            [MarshalAsAttribute(UnmanagedType.LPStr)] string lpsDevName, 
            [InAttribute()] 
            [MarshalAsAttribute(UnmanagedType.LPStr)] string lpsDevSerialId);

    // BOOL DAPI_GetSerialId(IN HANDLE hDevInstance, OUT LPCSTR lpsDevSerialId);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        ///lpsDevSerialId: LPCSTR->CHAR*
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_GetSerialId")]
        [return: MarshalAsAttribute(UnmanagedType.Bool)]
        public static extern bool GetSerialId(System.IntPtr hDevInstance,
            [OutAttribute()]
            [MarshalAsAttribute(UnmanagedType.LPStr)] System.Text.StringBuilder lpsDevSerialId);

    // BOOL DAPI_ConfigIoPorts(IN HANDLE hDevInstance, IN ULONG ulIoPortConfig);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        ///ulIoPortConfig: ULONG->unsigned int
        [DllImportAttribute("usbi2cio.dll", EntryPoint="DAPI_ConfigIoPorts")]
        [return: MarshalAsAttribute(UnmanagedType.Bool)]
        public static extern bool ConfigIoPorts(System.IntPtr hDevInstance, UInt32 ulIoPortConfig);

    // BOOL DAPI_GetIoConfig(IN HANDLE hDevInstance, OUT LPLONG lpulIoPortConfig);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        ///lpulIoPortConfig: LPLONG->int*
        [DllImportAttribute("usbi2cio.dll", EntryPoint="DAPI_GetIoConfig")]
        [return: MarshalAsAttribute(UnmanagedType.Bool)]
        public static extern bool GetIoConfig(System.IntPtr hDevInstance, out UInt32 lpulIoPortConfig);

    
    // BOOL DAPI_ReadIoPorts(IN HANDLE hDevInstance, OUT LPLONG lpulIoPortData);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        ///lpulIoPortData: LPLONG->int*
        [DllImportAttribute("usbi2cio.dll", EntryPoint="DAPI_ReadIoPorts")]
        [return: MarshalAsAttribute(UnmanagedType.Bool)]
        public static extern bool ReadIoPorts(System.IntPtr hDevInstance, out UInt32 lpulIoPortData) ;

    
    // BOOL DAPI_WriteIoPorts(IN HANDLE hDevInstance, IN ULONG ulIoPortData, IN ULONG ulIoPortMask);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        ///ulIoPortData: ULONG->unsigned int
        ///ulIoPortMask: ULONG->unsigned int
        [DllImportAttribute("usbi2cio.dll", EntryPoint="DAPI_WriteIoPorts")]
        [return: MarshalAsAttribute(UnmanagedType.Bool)]
        public static extern bool WriteIoPorts(System.IntPtr hDevInstance, UInt32 ulIoPortData, UInt32 ulIoPortMask) ;

    
    // LONG DAPI_BlockWriteIoPorts(IN HANDLE hDevInstance, IN WORD wBlockIoMode, IN WORD wBlockIoIndex, IN BYTE *pbyData, IN WORD wCount);

        /// Return Type: LONG->int
        ///hDevInstance: HANDLE->void*
        ///wBlockIoMode: WORD->unsigned short
        ///wBlockIoIndex: WORD->unsigned short
        ///pbyData: BYTE*
        ///wCount: WORD->unsigned short
        [DllImportAttribute("usbi2cio.dll", EntryPoint="DAPI_BlockWriteIoPorts")]
        public static extern Int32 BlockWriteIoPorts(System.IntPtr hDevInstance, UInt16 wBlockIoMode, UInt16 wBlockIoIndex, ref byte pbyData, UInt16 wCount) ;

    
    // LONG DAPI_BlockReadIoPorts(IN HANDLE hDevInstance, OUT BYTE *pbyData, IN WORD wBlockIoMode, IN WORD wBlockIoIndex, IN WORD wCount);

        /// Return Type: LONG->int
        ///hDevInstance: HANDLE->void*
        ///pbyData: BYTE*
        ///wBlockIoMode: WORD->unsigned short
        ///wBlockIoIndex: WORD->unsigned short
        ///wCount: WORD->unsigned short
        [DllImportAttribute("usbi2cio.dll", EntryPoint="DAPI_BlockReadIoPorts")]
        public static extern Int32 BlockReadIoPorts(System.IntPtr hDevInstance, ref byte pbyData, UInt16 wBlockIoMode, UInt16 wBlockIoIndex, UInt16 wCount);


    // LONG DAPI_ReadI2c(IN HANDLE hDevInstance, PI2C_TRANS TransI2C);

        /// Return Type: LONG->int
        ///hDevInstance: HANDLE->void*
        ///TransI2C: PI2C_TRANS->_I2C_TRANS*
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_ReadI2c")]
        public static extern Int32 ReadI2c(System.IntPtr hDevInstance, ref I2C_TRANS TransI2C);


    // LONG DAPI_WriteI2c(IN HANDLE hDevInstance, PI2C_TRANS TransI2C);

        /// Return Type: LONG->int
        ///hDevInstance: HANDLE->void*
        ///TransI2C: PI2C_TRANS->_I2C_TRANS*
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_WriteI2c")]
        public static extern Int32 WriteI2c(System.IntPtr hDevInstance, ref I2C_TRANS TransI2C);


    // BOOL  DAPI_SetProperty(IN HANDLE hDevInstance, IN BYTE byPropIndex, IN BYTE byPropValue);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        ///byPropIndex: BYTE->unsigned char
        ///byPropValue: BYTE->unsigned char
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_SetProperty")]
        [return: MarshalAsAttribute(UnmanagedType.Bool)]
        public static extern bool SetProperty(System.IntPtr hDevInstance, PROPERTY_IDX byPropIndex, byte byPropValue);


    // BOOL  DAPI_GetProperty(IN HANDLE hDevInstance, OUT LPBYTE lpbyPropValue, IN PROPERTY byPropIndex);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        ///lpbyPropValue: LPBYTE->BYTE*
        ///byPropIndex: BYTE->unsigned char
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_GetProperty")]
        [return: MarshalAsAttribute(UnmanagedType.Bool)]
        public static extern bool GetProperty(System.IntPtr hDevInstance, out byte lpbyPropValue, PROPERTY_IDX byPropIndex);

        // LONG DAPI_SetVendorRequest(IN HANDLE hDevInstance, IN BYTE byRequest, IN WORD wValue, IN WORD wIndex, IN WORD wLength, OUT PBYTE pbyGetData);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        ///byRequest: BYTE->unsigned char
        ///wValue: WORD->unsigned short
        ///wIndex: WORD->unsigned short
        ///wLength: WORD->unsigned short
        ///pbySetData: PBYTE->BYTE*
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_SetVendorRequest")]
        public static extern Int32 SetVendorRequest(System.IntPtr hDevInstance, byte byRequest, UInt16 wValue, UInt16 wIndex, UInt16 wLength, ref byte pbySetData);


        // LONG DAPI_GetVendorRequest(IN HANDLE hDevInstance, OUT PBYTE pbyGetData, IN BYTE byRequest, IN WORD wValue, IN WORD wIndex, IN WORD wLength);

        /// Return Type: LONG->int
        ///hDevInstance: HANDLE->void*
        ///pbyGetData: PBYTE->BYTE*
        ///byRequest: BYTE->unsigned char
        ///wValue: WORD->unsigned short
        ///wIndex: WORD->unsigned short
        ///wLength: WORD->unsigned short
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_GetVendorRequest")]
        public static extern Int32 GetVendorRequest(System.IntPtr hDevInstance, ref byte pbyGetData, byte byRequest, UInt16 wValue, UInt16 wIndex, UInt16 wLength);

        // BOOL  DAPI_GetLastFirmwareError(IN HANDLE hDevInstance, OUT LPLONG lplError);

        /// Return Type: BOOL->int
        ///hDevInstance: HANDLE->void*
        ///lplError: LPLONG->int*
        [DllImportAttribute("usbi2cio.dll", EntryPoint = "DAPI_GetLastFirmwareError")]
        [return: MarshalAsAttribute(UnmanagedType.Bool)]
        public static extern bool GetLastFirmwareError(System.IntPtr hDevInstance, out Int32 lplError);
    }
}
